@@ -113,8 +113,12 @@ class Terminal(RegionNode):
         Checks if a tree has any redundant nodes. 
         If yes, it returns False, else True.
         """
-        return True    
+        return True
 
+    def to_canon_tree(self):
+        node = deepcopy(self)
+        tree = Tree('||',[Tree('&',[node], level = 2)], level = 1)
+        return tree
     
 class Tree(RegionNode):
     def __init__(self, operator, operands, level = None):
@@ -148,12 +152,14 @@ class Tree(RegionNode):
         del self._operator          
 
     def __repr__(self):
+        return self.flatten()
         return '{t}({op}, {xyz})'.format(
             t=type(self).__name__,
             op=repr(self.operator),
             xyz=', '.join(repr(x) for x in self.operands))
 
     def __str__(self, depth=None):
+        return self.flatten_infix()
         if depth is not None:
             depth = depth - 1
         if depth == 0:
@@ -172,6 +178,12 @@ class Tree(RegionNode):
             self.opmap[self.operator],
             ', '.join(x.flatten() for x in self.operands),
             ')'])  
+
+    def flatten_infix(self):
+        return ' '.join([
+            '(',
+            '{} '.join(x.flatten() for x in self.operands).format(self.opmap[self.operator]),
+            ')'])  
     
     def insert_intersection(self):
         if self.operator == '&':
@@ -231,7 +243,7 @@ class Tree(RegionNode):
             tree = tree.extend_tree()
             tree = tree.lossless_transform()
             done = True
-        tree = tree.lossy_transform()
+        #tree = tree.lossy_transform()
         return tree
 
     def extend_tree(self):
@@ -450,9 +462,9 @@ class HalfSpace(Terminal):
         Terminal.__init__(self, approx)
         self.point = np.array(point)
         self.shape = self.point.shape
-        self.vector = np.array(vector)
-        self.b = np.dot(self.vector, self.point)
-        
+        self.vector = np.transpose(np.array(vector))
+        self.b = np.dot(self.vector, np.transpose(self.point))[0][0]
+
     @property
     def point(self):
         """I'm the 'coord' property."""
@@ -461,8 +473,11 @@ class HalfSpace(Terminal):
     @point.setter
     def point(self, value):
         if is_point(value):
+            self._point = np.transpose(value)
+        elif is_point(np.transpose(value)):
             self._point = value
         else:
+            print value
             print "The input must be a point, the given is {}".format(value)
             sys.exit(1)
 
@@ -478,19 +493,21 @@ class HalfSpace(Terminal):
     @vector.setter
     def vector(self, value):
         if is_point(value):
-            if value.shape == self.point.shape:
+            if value.shape[0] == self.point.shape[0] and value.shape[1] == self.point.shape[1]:
+                self._vector = value
+            elif value.shape[0] == self.point.shape[1] and value.shape[1] == self.point.shape[0] and value.shape[0]==1:
                 self._vector = value
             else:
                 print "The vector's dimensions {} doesn't agree with the"\
                 " point's dimensions {}".format(value.shape, self.point.shape)
                 sys.exit(1)
         else:
-            print "The input must be a point, the given is {}".format(value)
+            print "The input must be a vector, the given is {}".format(value)
             sys.exit(1)
 
     @vector.deleter
     def vector(self):
-        del self._point         
+        del self._vector        
         
     def __str__(self, depth=None):
         return 'HalfSpace ({}, {})'.format(self.point.tolist(), 
@@ -515,7 +532,21 @@ class Empty(Terminal):
 
     def flatten(self):
         return "{}".format(self.name)  
-        
+
+class Workspace(Terminal):
+    def __init__(self, approx=False):
+        Terminal.__init__(self, approx)
+        self.name = 'Workspace'
+
+    def __str__(self, depth=None):
+        return "Workspace()"                 
+
+    def __repr__(self):
+        return "{}".format(self.name)  
+
+    def flatten(self):
+        return "{}()".format(self.name)  
+       
 class Ellipsoid(Terminal):
     def __init__(self, P=None, xc=None, A=None, b=None, c=None, approx=False):
         Terminal.__init__(self, approx)
